@@ -65,11 +65,13 @@ struct _GdictAppletPrivate
   
   GConfClient *gconf_client;
   guint notify_id;
+  guint font_notify_id;
 
   gchar *database;
   gchar *strategy;
   gchar *source_name;  
   gchar *print_font;
+  gchar *defbox_font;
 
   gchar *word;  
   GdictContext *context;
@@ -84,6 +86,7 @@ struct _GdictAppletPrivate
   GtkWidget *image;
   GtkWidget *entry;
   GtkWidget *window;
+  GtkWidget *frame;
   GtkWidget *defbox;
 
   guint idle_draw_id;
@@ -96,6 +99,11 @@ struct _GdictAppletPrivate
   guint is_window_showing : 1;
 };
 
+#define WINDOW_MIN_WIDTH 	300
+#define WINDOW_MIN_HEIGHT 	200
+#define WINDOW_NUM_COLUMNS 	56
+#define WINDOW_NUM_ROWS  	23
+
 G_DEFINE_TYPE (GdictApplet, gdict_applet, PANEL_TYPE_APPLET);
 
 
@@ -128,7 +136,7 @@ show_error_dialog (GtkWindow   *parent,
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
   					      "%s", detail);
   
-  if (parent->group)
+  if (parent && parent->group)
     gtk_window_group_add_window (parent->group, GTK_WINDOW (dialog));
   
   gtk_dialog_run (GTK_DIALOG (dialog));
@@ -151,6 +159,50 @@ set_atk_name_description (GtkWidget  *widget,
   atk_object_set_description (aobj, description);
 }
 
+static void
+set_window_default_size (GdictApplet *applet)
+{
+  GdictAppletPrivate *priv = applet->priv;
+  GtkWidget *widget, *defbox;
+  gint width, height;
+  gint font_size;
+  GdkScreen *screen;
+  gint monitor_num;
+  GtkRequisition req;
+  GdkRectangle monitor;
+
+  if (!priv->window)
+    return;
+  
+  widget = priv->window;
+  defbox = priv->defbox;
+
+  /* Size based on the font size */
+  font_size = pango_font_description_get_size (defbox->style->font_desc);
+  font_size = PANGO_PIXELS (font_size);
+
+  width = font_size * WINDOW_NUM_COLUMNS;
+  height = font_size * WINDOW_NUM_ROWS;
+
+  /* Use at least the requisition size of the window... */
+  gtk_widget_size_request (widget, &req);
+  width = MAX (width, req.width);
+  height = MAX (height, req.height);
+
+  /* ... but make it no larger than the monitor */
+  screen = gtk_widget_get_screen (widget);
+  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+
+  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
+  width = MIN (width, monitor.width * 3 / 4);
+  height = MIN (height, monitor.height * 3 / 4);
+
+  /* Set size */
+  gtk_widget_set_size_request (priv->frame, width, height);
+
+}
+
 static void
 clear_cb (GtkWidget   *widget,
 	  GdictApplet *applet)
@@ -197,7 +249,7 @@ save_cb (GtkWidget   *widget,
   					NULL);
   gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
   
-  /* default to user's home */
+  /* default to user's $HOME */
   gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), g_get_home_dir ());
   gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog), _("Untitled document"));
   
@@ -269,12 +321,7 @@ gdict_applet_build_window (GdictApplet *applet)
   GtkWidget *frame;
   GtkWidget *vbox;
   GtkWidget *bbox;
-  GtkWidget *clear;
-  GtkWidget *print;
-  GtkWidget *save;
-  PangoContext *pango_context;
-  PangoFontDescription *font_desc;
-  gint width, height, font_size;
+  GtkWidget *button;
 
   if (!priv->entry)
     {
@@ -285,22 +332,11 @@ gdict_applet_build_window (GdictApplet *applet)
   
   window = gtk_aligned_window_new (priv->toggle);
 
-  /* compute the minimum size of the window pane depending on the
-   * size of the system font used to render the text.  this will
-   * be updated if the style changes
-   */
-  pango_context = gtk_widget_get_pango_context (window);
-  font_desc = pango_context_get_font_description (pango_context);
-  font_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
-
-  width = MAX (52 * font_size, 330);
-  height = MAX (24 * font_size, 220);
-  
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
-  gtk_widget_set_size_request (frame, width, height);
   gtk_container_add (GTK_CONTAINER (window), frame);
   gtk_widget_show (frame);
+  priv->frame = frame;
   
   vbox = gtk_vbox_new (FALSE, 12);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
@@ -320,47 +356,49 @@ gdict_applet_build_window (GdictApplet *applet)
   gtk_box_pack_end (GTK_BOX (vbox), bbox, FALSE, FALSE, 0);
   gtk_widget_show (bbox);
   
-  clear = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
+  button = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
 
   gtk_tooltips_set_tip (priv->tooltips,
-		  	clear,
+		  	button,
 		  	_("Clear the definitions found"),
 			NULL);
-  set_atk_name_description (clear,
+  set_atk_name_description (button,
 		  	    _("Clear definition"),
 			    _("Clear the text of the definition"));
   
-  g_signal_connect (clear, "clicked", G_CALLBACK (clear_cb), applet);
-  gtk_box_pack_start (GTK_BOX (bbox), clear, FALSE, FALSE, 0);
-  gtk_widget_show (clear);
+  g_signal_connect (button, "clicked", G_CALLBACK (clear_cb), applet);
+  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
 
-  print = gtk_button_new_from_stock (GTK_STOCK_PRINT);
+  button = gtk_button_new_from_stock (GTK_STOCK_PRINT);
 
   gtk_tooltips_set_tip (priv->tooltips,
-		  	print,
+		  	button,
 			_("Print the definitions found"),
 			NULL);
-  set_atk_name_description (print,
+  set_atk_name_description (button,
 		  	    _("Print definition"),
 			    _("Print the text of the definition"));
   
-  g_signal_connect (print, "clicked", G_CALLBACK (print_cb), applet);
-  gtk_box_pack_start (GTK_BOX (bbox), print, FALSE, FALSE, 0);
-  gtk_widget_show (print);
+  g_signal_connect (button, "clicked", G_CALLBACK (print_cb), applet);
+  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
 
-  save = gtk_button_new_from_stock (GTK_STOCK_SAVE);
+  button = gtk_button_new_from_stock (GTK_STOCK_SAVE);
   
   gtk_tooltips_set_tip (priv->tooltips,
-		  	save,
+		  	button,
 			_("Save the definitions found"),
 			NULL);
-  set_atk_name_description (save,
+  set_atk_name_description (button,
 		  	    _("Save definition"),
 			    _("Save the text of the definition to a file"));
   
-  g_signal_connect (save, "clicked", G_CALLBACK (save_cb), applet);
-  gtk_box_pack_start (GTK_BOX (bbox), save, FALSE, FALSE, 0);
-  gtk_widget_show (save);
+  g_signal_connect (button, "clicked", G_CALLBACK (save_cb), applet);
+  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+  
+  set_window_default_size (applet);
   
   priv->window = window;
   priv->is_window_showing = FALSE;
@@ -810,35 +848,11 @@ static void
 gdict_applet_style_set (GtkWidget *widget,
 			GtkStyle  *old_style)
 {
-  GdictApplet *applet;
-  GdictAppletPrivate *priv;
-  PangoContext *pango_context;
-  PangoFontDescription *font_desc;
-  gint font_size, width, height;
-  GtkWidget *frame;
-  
   if (GTK_WIDGET_CLASS (gdict_applet_parent_class)->style_set)
     GTK_WIDGET_CLASS (gdict_applet_parent_class)->style_set (widget,
 		    					     old_style);
 
-  applet = GDICT_APPLET (widget);
-  priv = applet->priv;
-
-  if (!priv->window)
-    return;
-
-  frame = GTK_BIN (priv->window)->child;
-  if (!frame)
-    return;
-
-  pango_context = gtk_widget_get_pango_context (priv->window);
-  font_desc = pango_context_get_font_description (pango_context);
-  font_size = pango_font_description_get_size (font_desc) / PANGO_SCALE;
-
-  width = MAX (52 * font_size, 330);
-  height = MAX (24 * font_size, 220);
-
-  gtk_widget_set_size_request (frame, width, height);
+  set_window_default_size (GDICT_APPLET (widget));
 }
 
 static void
@@ -958,6 +972,32 @@ gdict_applet_set_print_font (GdictApplet *applet,
   priv->print_font = g_strdup (print_font);
 }
 
+static void
+gdict_applet_set_defbox_font (GdictApplet *applet,
+			      const gchar *defbox_font)
+{
+  GdictAppletPrivate *priv = applet->priv;
+  
+  if (!defbox_font)
+    defbox_font = gconf_client_get_string (priv->gconf_client,
+		    			   DOCUMENT_FONT_KEY,
+					   NULL);
+
+  if (!defbox_font)
+    defbox_font = GDICT_DEFAULT_DEFBOX_FONT;
+
+  if (priv->defbox_font)
+    g_free (priv->defbox_font);
+
+  priv->defbox_font = g_strdup (defbox_font);
+
+  if (!priv->defbox)
+    return;
+
+  gdict_defbox_set_font_name (GDICT_DEFBOX (priv->defbox),
+			      priv->defbox_font);
+}
+
 static void
 gdict_applet_set_word (GdictApplet *applet,
 		       const gchar *word)
@@ -1073,6 +1113,13 @@ gdict_applet_gconf_notify_cb (GConfClient *client,
       else
         gdict_applet_set_strategy (applet, GDICT_DEFAULT_STRATEGY);
     }
+  else if (strcmp (entry->key, DOCUMENT_FONT_KEY) == 0)
+    {
+      if (entry->value && (entry->value->type == GCONF_VALUE_STRING))
+        gdict_applet_set_defbox_font (applet, gconf_value_get_string (entry->value));
+      else
+        gdict_applet_set_defbox_font (applet, GDICT_DEFAULT_DEFBOX_FONT);
+    }
 }
 
 static void
@@ -1080,6 +1127,15 @@ gdict_applet_finalize (GObject *object)
 {
   GdictApplet *applet = GDICT_APPLET (object);
   GdictAppletPrivate *priv = applet->priv;
+
+  if (priv->idle_draw_id)
+    g_source_remove (priv->idle_draw_id);
+
+  if (priv->notify_id)
+    gconf_client_notify_remove (priv->gconf_client, priv->notify_id);
+
+  if (priv->font_notify_id)
+    gconf_client_notify_remove (priv->gconf_client, priv->font_notify_id);
   
   if (priv->gconf_client)
     g_object_unref (priv->gconf_client);
@@ -1107,6 +1163,7 @@ gdict_applet_finalize (GObject *object)
   
   g_free (priv->source_name);
   g_free (priv->print_font);
+  g_free (priv->defbox_font);
   g_free (priv->word);
   
   G_OBJECT_CLASS (gdict_applet_parent_class)->finalize (object);
@@ -1134,27 +1191,13 @@ static void
 gdict_applet_init (GdictApplet *applet)
 {
   GdictAppletPrivate *priv;
-  gchar *data_dir, *icon_file;
+  gchar *icon_file, *data_dir;
   GdkPixbuf *icon;
-  GError *icon_error;
+  GError *icon_error, *gconf_error;
 
   priv = GDICT_APPLET_GET_PRIVATE (applet);
   applet->priv = priv;
       
-  /* create the data directory inside $HOME, if it doesn't exist yet */
-  data_dir = g_build_filename (g_get_home_dir (),
-  			       ".gnome2",
-  			       "gnome-dictionary",
-  			       NULL);
-
-  if (g_mkdir (data_dir, 0700) == -1)
-    {
-      if (errno != EEXIST)
-	show_error_dialog (NULL,
-			   _("Unable to create the data directory '%s'"),
-			   NULL);
-    }
-  
   icon_file = g_build_filename (DATADIR,
 		  		"pixmaps",
 				"gnome-dictionary.png",
@@ -1185,19 +1228,59 @@ gdict_applet_init (GdictApplet *applet)
   if (!priv->gconf_client)
     priv->gconf_client = gconf_client_get_default ();
   
+  gconf_error = NULL;
   gconf_client_add_dir (priv->gconf_client,
   			GDICT_GCONF_DIR,
   			GCONF_CLIENT_PRELOAD_ONELEVEL,
-  			NULL);
-  gconf_client_notify_add (priv->gconf_client,
-		  	   GDICT_GCONF_DIR,
-			   (GConfClientNotifyFunc) gdict_applet_gconf_notify_cb,
-			   applet, NULL,
-			   NULL);
+  			&gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to connect to GConf"),
+			 gconf_error->message);
+      
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
+  
+  priv->notify_id = gconf_client_notify_add (priv->gconf_client,
+		  			     GDICT_GCONF_DIR,
+					     gdict_applet_gconf_notify_cb,
+					     applet, NULL,
+					     &gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+		         _("Unable to get notification for preferences"),
+			 gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
+  
+  priv->font_notify_id =  gconf_client_notify_add (priv->gconf_client,
+		  				   DOCUMENT_FONT_KEY,
+						   gdict_applet_gconf_notify_cb,
+						   applet, NULL,
+						   &gconf_error);
+  if (gconf_error)
+    {
+      show_error_dialog (NULL,
+			 _("Unable to get notification for the document font"),
+			 gconf_error->message);
+
+      g_error_free (gconf_error);
+      gconf_error = NULL;
+    }
   
   if (!priv->loader)
     priv->loader = gdict_source_loader_new ();
 
+  data_dir = g_build_filename (g_get_home_dir (),
+		  	       ".gnome2",
+			       "gnome-dictionary",
+			       NULL);
+
   /* add our data dir inside $HOME to the loader's search paths */
   gdict_source_loader_add_search_path (priv->loader, data_dir);
 
@@ -1233,7 +1316,10 @@ gdict_applet_init (GdictApplet *applet)
   /* force first draw */
   gdict_applet_draw (applet);
 
+  /* force retrieval of the configuration from GConf */
   gdict_applet_set_source_name (applet, NULL);
+  gdict_applet_set_defbox_font (applet, NULL);
+  gdict_applet_set_print_font (applet, NULL);
 }
 
 #ifndef GDICT_APPLET_STAND_ALONE
@@ -1280,6 +1366,101 @@ gdict_applet_factory (PanelApplet *applet,
 }
 #endif /* !GDICT_APPLET_STAND_ALONE */
 
+/* create the data directory inside $HOME, if it doesn't exist yet */
+static gboolean
+gdict_applet_create_data_dir (void)
+{
+  gchar *data_dir_name;
+  
+  data_dir_name = g_build_filename (g_get_home_dir (),
+  				    ".gnome2",
+  				    "gnome-dictionary",
+  				    NULL);
+  
+  if (g_mkdir (data_dir_name, 0700) == -1)
+    {
+      /* this is weird, but sometimes there's a "gnome-dictionary" file
+       * inside $HOME/.gnome2; see bug #329126.
+       */
+      if ((errno == EEXIST) &&
+          (g_file_test (data_dir_name, G_FILE_TEST_IS_REGULAR)))
+        {
+          gchar *backup = g_strdup_printf ("%s.pre-2-14", data_dir_name);
+	      
+	  if (g_rename (data_dir_name, backup) == -1)
+	    {
+              GtkWidget *error_dialog;
+
+	      error_dialog = gtk_message_dialog_new (NULL,
+                                                     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("Unable to rename file '%s' to '%s': %s"),
+						     data_dir_name,
+						     backup,
+						     g_strerror (errno));
+	      
+	      gtk_dialog_run (GTK_DIALOG (error_dialog));
+	      
+	      gtk_widget_destroy (error_dialog);
+	      g_free (backup);
+	      g_free (data_dir_name);
+
+	      return FALSE;
+            }
+
+	  g_free (backup);
+	  
+          if (g_mkdir (data_dir_name, 0700) == -1)
+            {
+              GtkWidget *error_dialog;
+	      
+	      error_dialog = gtk_message_dialog_new (NULL,
+						     GTK_DIALOG_MODAL,
+						     GTK_MESSAGE_ERROR,
+						     GTK_BUTTONS_CLOSE,
+						     _("Unable to create the data directory '%s': %s"),
+						     data_dir_name,
+						     g_strerror (errno));
+	      
+	      gtk_dialog_run (GTK_DIALOG (error_dialog));
+	      
+	      gtk_widget_destroy (error_dialog);
+              g_free (data_dir_name);
+
+	      return FALSE;
+            }
+
+	  goto success;
+	}
+      
+      if (errno != EEXIST)
+        {
+          GtkWidget *error_dialog;
+
+	  error_dialog = gtk_message_dialog_new (NULL,
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("Unable to create the data directory '%s': %s"),
+						 data_dir_name,
+						 g_strerror (errno));
+	  
+	  gtk_dialog_run (GTK_DIALOG (error_dialog));
+	  
+	  gtk_widget_destroy (error_dialog);
+	  g_free (data_dir_name);
+
+	  return FALSE;
+	}
+    }
+
+success:
+  g_free (data_dir_name);
+
+  return TRUE;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -1303,6 +1484,9 @@ main (int argc, char *argv[])
 		      GNOME_PROGRAM_STANDARD_PROPERTIES,
 		      NULL);
 
+  if (!gdict_applet_create_data_dir ())
+    return FALSE;
+
   return panel_applet_factory_main ("OAFIID:GNOME_DictionaryApplet_Factory",
 		                    GDICT_TYPE_APPLET,
 				    gdict_applet_factory,
@@ -1310,6 +1494,9 @@ main (int argc, char *argv[])
 #else
   gtk_init (&argc, &argv);
 
+  if (!gdict_applet_create_data_dir ())
+    return FALSE;
+  
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);
 
