@@ -54,8 +54,6 @@ struct _LogviewWindow {
         GSList *logs;
 	Log *curlog;
 
-	GtkClipboard *clipboard;
-	
 	int original_fontsize, fontsize;
 };
 
@@ -73,6 +71,7 @@ void logview_add_log_from_name (LogviewWindow *logview, gchar *file);
 void logview_add_logs_from_names (LogviewWindow *logview, GSList *lognames, gchar *selected);
 void logview_menus_set_state (LogviewWindow *logview);
 void logview_set_window_title (LogviewWindow *window);
+void logview_set_font (LogviewWindow *window, const gchar *fontname);
 void logview_show_main_content (LogviewWindow *window);
 GtkWidget *logview_window_new (void);
 
